@@ -58,6 +58,7 @@ from _ast import (
     )
 
 from logilab.astng import nodes as new
+import sys
 
 _BIN_OP_CLASSES = {_Add: '+',
                    _BitAnd: '&',
@@ -107,45 +108,6 @@ def _init_set_doc(node, newnode):
 
     except IndexError:
         pass # ast built from scratch
-    
-
-
-def native_repr_tree(node, indent='', _done=None):
-    if _done is None:
-        _done = set()
-    if node in _done:
-        print ('loop in tree: %r (%s)' % (node, getattr(node, 'lineno', None)))
-        return
-    _done.add(node)
-    print indent + str(node)
-    if type(node) is str: # XXX crash on Globals
-        return
-    indent += '    '
-    d = node.__dict__
-    if hasattr(node, '_attributes'):
-        for a in node._attributes:
-            attr = d[a]
-            if attr is None:
-                continue
-            print indent + a, repr(attr)
-    for f in node._fields or ():
-        attr = d[f]
-        if attr is None:
-            continue
-        if type(attr) is list:
-            if not attr: continue
-            print indent + f + ' ['
-            for elt in attr:
-                native_repr_tree(elt, indent, _done)
-            print indent + ']'
-            continue
-        if isinstance(attr, (_Load, _Store, _Del)):
-            continue
-        if isinstance(attr, Node):
-            print indent + f
-            native_repr_tree(attr, indent, _done)
-        else:
-            print indent + f, repr(attr)
 
 
 from logilab.astng.rebuilder import RebuildVisitor
@@ -212,6 +174,14 @@ class TreeRebuilder(RebuildVisitor):
         newnode.set_line_info(newnode.last_child())
         return newnode
 
+    def visit_assname(self, node, parent, node_name=None):
+        '''visit a node and return a AssName node'''
+        newnode = new.AssName()
+        self._set_infos(node, newnode, parent)
+        newnode.name = node_name
+        self._save_assignment(newnode)
+        return newnode
+
     def visit_augassign(self, node, parent):
         """visit a AugAssign node by returning a fresh instance of it"""
         newnode = new.AugAssign()
@@ -270,6 +240,8 @@ class TreeRebuilder(RebuildVisitor):
         _init_set_doc(node, newnode)
         newnode.bases = [self.visit(child, newnode) for child in node.bases]
         newnode.body = [self.visit(child, newnode) for child in node.body]
+        if 'decorator_list' in node._fields and node.decorator_list:# py >= 2.6
+            newnode.decorators = self.visit_decorators(node, newnode)
         newnode.set_line_info(newnode.last_child())
         return newnode
 
@@ -328,6 +300,17 @@ class TreeRebuilder(RebuildVisitor):
         newnode.set_line_info(newnode.last_child())
         return newnode
 
+    def visit_dictcomp(self, node, parent):
+        """visit a DictComp node by returning a fresh instance of it"""
+        newnode = new.DictComp()
+        _lineno_parent(node, newnode, parent)
+        newnode.key = self.visit(node.key, newnode)
+        newnode.value = self.visit(node.value, newnode)
+        newnode.generators = [self.visit(child, newnode)
+                              for child in node.generators]
+        newnode.set_line_info(newnode.last_child())
+        return newnode
+
     def visit_discard(self, node, parent):
         """visit a Discard node by returning a fresh instance of it"""
         newnode = new.Discard()
@@ -341,9 +324,11 @@ class TreeRebuilder(RebuildVisitor):
         newnode = new.ExceptHandler()
         _lineno_parent(node, newnode, parent)
         newnode.type = self.visit(node.type, newnode)
-        self.asscontext = "Ass"
-        newnode.name = self.visit(node.name, newnode)
-        self.asscontext = None
+        if node.name is not None:
+            # /!\ node.name can be a tuple
+            self.asscontext = "Ass"
+            newnode.name = self.visit(node.name, newnode)
+            self.asscontext = None
         newnode.body = [self.visit(child, newnode) for child in node.body]
         newnode.set_line_info(newnode.last_child())
         return newnode
@@ -382,7 +367,7 @@ class TreeRebuilder(RebuildVisitor):
     def visit_from(self, node, parent):
         """visit a From node by returning a fresh instance of it"""
         names = [(alias.name, alias.asname) for alias in node.names]
-        newnode = new.From(node.module, names, node.level)
+        newnode = new.From(node.module or '', names, node.level)
         self._set_infos(node, newnode, parent)
         self._store_from_node(newnode)
 
@@ -402,8 +387,6 @@ class TreeRebuilder(RebuildVisitor):
         decorators = getattr(node, attr)
         if decorators:
             newnode.decorators = self.visit_decorators(node, newnode)
-        else:
-            newnode.decorators = None
         newnode.set_line_info(newnode.last_child())
         return newnode
 
@@ -508,9 +491,10 @@ class TreeRebuilder(RebuildVisitor):
         newnode.set_line_info(newnode.last_child())
         return newnode
 
-    def visit_module(self, node, modname):
+    def visit_module(self, node, modname, package):
         """visit a Module node by returning a fresh instance of it"""
         newnode = new.Module(modname, None)
+        newnode.package = package
         _lineno_parent(node, newnode, parent=None)
         _init_set_doc(node, newnode)
         newnode.body = [self.visit(child, newnode) for child in node.body]
@@ -538,14 +522,20 @@ class TreeRebuilder(RebuildVisitor):
         newnode.set_line_info(newnode.last_child())
         return newnode
 
+    def visit_bytes(self, node, parent):
+        """visit a Bytes node by returning a fresh instance of Const"""
+        newnode = new.Const(node.s)
+        self._set_infos(node, newnode, parent)
+        return newnode
+
     def visit_num(self, node, parent):
-        """visit a a Num node by returning a fresh instance of Const"""
+        """visit a Num node by returning a fresh instance of Const"""
         newnode = new.Const(node.n)
         self._set_infos(node, newnode, parent)
         return newnode
 
     def visit_str(self, node, parent):
-        """visit a a Str node by returning a fresh instance of Const"""
+        """visit a Str node by returning a fresh instance of Const"""
         newnode = new.Const(node.s)
         self._set_infos(node, newnode, parent)
         return newnode
@@ -564,7 +554,7 @@ class TreeRebuilder(RebuildVisitor):
         """visit a Raise node by returning a fresh instance of it"""
         newnode = new.Raise()
         _lineno_parent(node, newnode, parent)
-        newnode.type = self.visit(node.type, newnode)
+        newnode.exc = self.visit(node.type, newnode)
         newnode.inst = self.visit(node.inst, newnode)
         newnode.tback = self.visit(node.tback, newnode)
         newnode.set_line_info(newnode.last_child())
@@ -578,6 +568,24 @@ class TreeRebuilder(RebuildVisitor):
         newnode.set_line_info(newnode.last_child())
         return newnode
 
+    def visit_set(self, node, parent):
+        """visit a Tuple node by returning a fresh instance of it"""
+        newnode = new.Set()
+        _lineno_parent(node, newnode, parent)
+        newnode.elts = [self.visit(child, newnode) for child in node.elts]
+        newnode.set_line_info(newnode.last_child())
+        return newnode
+
+    def visit_setcomp(self, node, parent):
+        """visit a SetComp node by returning a fresh instance of it"""
+        newnode = new.SetComp()
+        _lineno_parent(node, newnode, parent)
+        newnode.elt = self.visit(node.elt, newnode)
+        newnode.generators = [self.visit(child, newnode)
+                              for child in node.generators]
+        newnode.set_line_info(newnode.last_child())
+        return newnode
+
     def visit_slice(self, node, parent):
         """visit a Slice node by returning a fresh instance of it"""
         newnode = new.Slice()
@@ -665,3 +673,53 @@ class TreeRebuilder(RebuildVisitor):
         newnode.set_line_info(newnode.last_child())
         return newnode
 
+
+class TreeRebuilder3k(TreeRebuilder):
+    """extend and overwrite TreeRebuilder for python3k"""
+
+    def visit_arg(self, node, parent):
+        """visit a arg node by returning a fresh AssName instance"""
+        # the <arg> node is coming from py>=3.0, but we use AssName in py2.x
+        # XXX or we should instead introduce a Arg node in astng ?
+        return self.visit_assname(node, parent, node.arg)
+
+    def visit_excepthandler(self, node, parent):
+        """visit an ExceptHandler node by returning a fresh instance of it"""
+        newnode = new.ExceptHandler()
+        _lineno_parent(node, newnode, parent)
+        newnode.type = self.visit(node.type, newnode)
+        if node.name is not None:
+            newnode.name = self.visit_assname(node, newnode, node.name)
+        newnode.body = [self.visit(child, newnode) for child in node.body]
+        newnode.set_line_info(newnode.last_child())
+        return newnode
+
+    def visit_nonlocal(self, node, parent):
+        """visit a Nonlocal node and return a new instance of it"""
+        newnode = new.Nonlocal(node.names)
+        self._set_infos(node, newnode, parent)
+        return newnode
+
+    def visit_raise(self, node, parent):
+        """visit a Raise node by returning a fresh instance of it"""
+        newnode = new.Raise()
+        _lineno_parent(node, newnode, parent)
+        # no traceback; anyway it is not used in Pylint
+        newnode.exc = self.visit(node.exc, newnode)
+        newnode.cause = self.visit(node.cause, newnode)
+        newnode.set_line_info(newnode.last_child())
+        return newnode
+
+    def visit_starred(self, node, parent):
+        """visit a Starred node and return a new instance of it"""
+        newnode = new.Starred()
+        _lineno_parent(node, newnode, parent)
+        newnode.value = self.visit(node.value, newnode)
+        newnode.set_line_info(newnode.last_child())
+        return newnode
+
+
+if sys.version_info >= (3, 0):
+    TreeRebuilder = TreeRebuilder3k
+
+
