@@ -50,10 +50,12 @@ except ImportError:
     class BaseClass:
         pass
 
-from logilab.common.compat import set
-from logilab.astng._exceptions import (InferenceError, ASTNGError,
-                                       NotFoundError, UnresolvableName)
+from logilab.common.compat import builtins
+from logilab.astng._exceptions import InferenceError, ASTNGError, \
+                                       NotFoundError, UnresolvableName
+from logilab.astng.as_string import as_string
 
+BUILTINS_NAME = builtins.__name__
 
 class Proxy(BaseClass):
     """a simple proxy object"""
@@ -168,7 +170,7 @@ class Instance(Proxy):
             if lookupclass:
                 # class attributes not available through the instance
                 # unless they are explicitly defined
-                if name in ('__name__', '__bases__', '__mro__'):
+                if name in ('__name__', '__bases__', '__mro__', '__subclasses__'):
                     return self._proxied.local_attr(name)
                 return self._proxied.getattr(name, context)
             raise NotFoundError(name)
@@ -340,6 +342,8 @@ class NodeNG(BaseClass):
     It represents a node of the new abstract syntax tree.
     """
     is_statement = False
+    optional_assign = False # True  for For (and for Comprehension if py <3.0)
+    is_function = False # True for Function nodes
     # attributes below are set by the builder module or by raw factories
     lineno = None
     fromlineno = None
@@ -370,9 +374,8 @@ class NodeNG(BaseClass):
         return func(self)
 
     def get_children(self):
-        node_dict = self.__dict__
         for field in self._astng_fields:
-            attr = node_dict[field]
+            attr = getattr(self, field)
             if attr is None:
                 continue
             if isinstance(attr, (list, tuple)):
@@ -383,9 +386,8 @@ class NodeNG(BaseClass):
 
     def last_child(self):
         """an optimized version of list(get_children())[-1]"""
-        n_dict = self.__dict__
         for field in self._astng_fields[::-1]:
-            attr = n_dict[field]
+            attr = getattr(self, field)
             if not attr: # None or empty listy / tuple
                 continue
             if isinstance(attr, (list, tuple)):
@@ -561,14 +563,15 @@ class NodeNG(BaseClass):
         return False
 
     def as_string(self):
-        from logilab.astng.nodes_as_string import as_string
         return as_string(self)
 
     def repr_tree(self, ids=False):
-        """print a nice astng tree representation"""
+        """print a nice astng tree representation.
+
+        :param ids: if true, we also print the ids (usefull for debugging)"""
         result = []
         _repr_tree(self, result, ids=ids)
-        print "\n".join(result)
+        return "\n".join(result)
 
 
 INDENT = "    "
