@@ -1,15 +1,3 @@
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU Lesser General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # copyright 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
 # copyright 2003-2010 Sylvain Thenault, all rights reserved.
@@ -35,8 +23,6 @@ With python >= 2.5, the internal _ast module is used instead
 
 The builder is not thread safe and can't be used to parse different sources
 at the same time.
-
-
 """
 
 __docformat__ = "restructuredtext en"
@@ -47,21 +33,18 @@ from inspect import isfunction, ismethod, ismethoddescriptor, isclass, \
      isbuiltin
 from inspect import isdatadescriptor
 
-from logilab.common.fileutils import norm_read
 from logilab.common.modutils import modpath_from_file
 
 from logilab.astng._exceptions import ASTNGBuildingException
-from logilab.astng.raw_building import *
+from logilab.astng.raw_building import build_module, object_build_class, \
+     object_build_function, object_build_datadescriptor, attach_dummy_node, \
+     object_build_methoddescriptor, attach_const_node, attach_import_node
+from logilab.astng.manager import ASTNGManager
 
-try:
-    from _ast import PyCF_ONLY_AST
-    def parse(string):
-        return compile(string, "<string>", 'exec', PyCF_ONLY_AST)
-    from logilab.astng._nodes_ast import TreeRebuilder
-except ImportError, exc:
-    from compiler import parse
-    from logilab.astng import patchcomptransformer
-    from logilab.astng._nodes_compiler import TreeRebuilder
+from _ast import PyCF_ONLY_AST
+def parse(string):
+    return compile(string, "<string>", 'exec', PyCF_ONLY_AST)
+from logilab.astng._nodes_ast import TreeRebuilder
 
 # ast NG builder ##############################################################
 
@@ -71,10 +54,9 @@ class ASTNGBuilder:
 
     def __init__(self, manager=None):
         if manager is None:
-            from logilab.astng import MANAGER as manager
+            manager = ASTNGManager()
         self._manager = manager
         self._module = None
-        self._file = None
         self._done = None
         self.rebuilder = TreeRebuilder(manager)
         self._dyn_modname_map = {'gtk': 'gtk._gtk'}
@@ -118,11 +100,10 @@ class ASTNGBuilder:
         path is expected to be a python source file
         """
         try:
-            data = norm_read(path)
+            data = open(path, 'U').read()
         except IOError, ex:
             msg = 'Unable to load file %r (%s)' % (path, ex)
             raise ASTNGBuildingException(msg)
-        self._file = path
         # get module name if necessary, *before modifying sys.path*
         if modname is None:
             try:
@@ -133,9 +114,7 @@ class ASTNGBuilder:
         try:
             sys.path.insert(0, dirname(path))
             node = self.string_build(data, modname, path)
-            node.file = abspath(path)
         finally:
-            self._file = None
             sys.path.pop(0)
 
         return node
@@ -155,8 +134,7 @@ class ASTNGBuilder:
             package = True
         else:
             package = path and path.find('__init__.py') > -1 or False
-        newnode = self.rebuilder.build(node, modname, node_file)
-        newnode.package = package
+        newnode = self.rebuilder.build(node, modname, node_file, package)
         return newnode
 
     # astng from living objects ###############################################
@@ -168,7 +146,7 @@ class ASTNGBuilder:
         """recursive method which create a partial ast from real objects
          (only function, class, and method are handled)
         """
-        if self._done.has_key(obj):
+        if obj in self._done:
             return self._done[obj]
         self._done[obj] = node
         for name in dir(obj):
@@ -205,6 +183,8 @@ class ASTNGBuilder:
                     class_node = object_build_class(node, member, name)
                     # recursion
                     self.object_build(class_node, member)
+                if name == '__class__' and class_node.parent is None:
+                    class_node.parent = self._done[self._module]
             elif ismethoddescriptor(member):
                 assert isinstance(member, object)
                 object_build_methoddescriptor(node, member, name)
