@@ -1,15 +1,3 @@
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU Lesser General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # copyright 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
 # copyright 2003-2010 Sylvain Thenault, all rights reserved.
@@ -30,27 +18,30 @@
 # You should have received a copy of the GNU Lesser General Public License along
 # with logilab-astng. If not, see <http://www.gnu.org/licenses/>.
 """this module contains a set of functions to handle inference on astng trees
-
 """
 
 from __future__ import generators
 
 __doctype__ = "restructuredtext en"
 
-from logilab.common.compat import chain
+from itertools import chain
+import sys
 
 try:
     GeneratorExit # introduced in py2.5
 except NameError:
     class GeneratorExit(Exception): pass
 
-from logilab.astng import MANAGER, nodes, raw_building
+from logilab.astng import nodes, raw_building
+from logilab.astng.manager import ASTNGManager
 from logilab.astng import ASTNGError, InferenceError, UnresolvableName, \
      NoDefault, NotFoundError, ASTNGBuildingException
 from logilab.astng.bases import YES, Instance, InferenceContext, \
      _infer_stmts, copy_context, path_wrapper, raise_if_nothing_infered
 from logilab.astng.protocols import _arguments_infer_argname
 
+MANAGER = ASTNGManager()
+
 _CONST_PROXY = {
     type(None): raw_building.build_class('NoneType'),
     bool: MANAGER.astng_from_class(bool),
@@ -62,6 +53,8 @@ _CONST_PROXY = {
     unicode: MANAGER.astng_from_class(unicode),
     }
 _CONST_PROXY[type(None)].parent = _CONST_PROXY[bool].parent
+if sys.version_info >= (2, 6):
+    _CONST_PROXY[bytes] = MANAGER.astng_from_class(bytes)
 
 # TODO : find a nicer way to handle this situation; we should at least
 # be able to avoid calling MANAGER.astng_from_class(const.value.__class__)
