@@ -1,16 +1,3 @@
-# -*- coding: utf-8 -*-
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU Lesser General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # copyright 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
 # copyright 2003-2010 Sylvain Thenault, all rights reserved.
@@ -30,32 +17,34 @@
 #
 # You should have received a copy of the GNU Lesser General Public License along
 # with logilab-astng. If not, see <http://www.gnu.org/licenses/>.
+"""Module for some node classes. More nodes in scoped_nodes.py
 """
-Module for some node classes. More nodes in scoped_nodes.py
 
-
-"""
-from logilab.common.compat import chain, imap
+import sys
 
 from logilab.astng import NoDefault
-from logilab.astng.bases import (NodeNG, BaseClass, Instance, copy_context,
-                                _infer_stmts, YES)
-from logilab.astng.mixins import (StmtMixIn, BlockRangeMixIn, AssignTypeMixin,
-    ParentAssignTypeMixin, FromImportMixIn)
+from logilab.astng.bases import NodeNG, BaseClass, Instance, copy_context, \
+                                _infer_stmts, YES
+from logilab.astng.mixins import StmtMixIn, BlockRangeMixIn, AssignTypeMixin, \
+                                 ParentAssignTypeMixin, FromImportMixIn
+
 
 def unpack_infer(stmt, context=None):
-    """return an iterator on nodes inferred by the given statement if the
-    inferred value is a list or a tuple, recurse on it to get values inferred
-    by its content
+    """recursively generate nodes inferred by the given statement.
+    If the inferred value is a list or a tuple, recurse on the elements
     """
     if isinstance(stmt, (List, Tuple)):
-        # XXX loosing context
-        return chain(*imap(unpack_infer, stmt.elts))
+        for elt in stmt.elts:
+            for infered_elt in unpack_infer(elt, context):
+                yield infered_elt
+        return
     infered = stmt.infer(context).next()
-    if infered is stmt:
-        return iter( (stmt,) )
-    return chain(*imap(unpack_infer, stmt.infer(context)))
-
+    if infered is stmt or infered is YES:
+        yield infered
+        return
+    for infered in stmt.infer(context):
+        for inf_inf in unpack_infer(infered, context):
+            yield inf_inf
 
 
 def are_exclusive(stmt1, stmt2, exceptions=None):
@@ -85,7 +74,7 @@ def are_exclusive(stmt1, stmt2, exceptions=None):
     node = stmt2.parent
     previous = stmt2
     while node:
-        if stmt1_parents.has_key(node):
+        if node in stmt1_parents:
             # if the common parent is a If or TryExcept statement, look if
             # nodes are in exclusive branches
             if isinstance(node, If) and exceptions is None:
@@ -109,9 +98,6 @@ def are_exclusive(stmt1, stmt2, exceptions=None):
     return False
 
 
-
-
-
 class LookupMixIn(BaseClass):
     """Mixin looking up a name in the right scope
     """
@@ -187,7 +173,7 @@ class LookupMixIn(BaseClass):
             if done:
                 break
 
-            optional_assign = isinstance(ass_type, (For, Comprehension))
+            optional_assign = ass_type.optional_assign
             if optional_assign and ass_type.parent_of(self):
                 # we are inside a loop, loop var assigment is hidding previous
                 # assigment
@@ -422,6 +408,7 @@ class Comprehension(NodeNG):
     iter = None
     ifs = None
 
+    optional_assign = True
     def ass_type(self):
         return self
 
@@ -585,6 +572,7 @@ class For(BlockRangeMixIn, StmtMixIn, AssignTypeMixin, NodeNG):
     body = None
     orelse = None
 
+    optional_assign = True
     def _blockstart_toline(self):
         return self.iter.tolineno
 
@@ -672,12 +660,14 @@ class List(NodeNG, Instance, ParentAssignTypeMixin):
         return self.elts
 
 
+class Nonlocal(StmtMixIn, NodeNG):
+    """class representing a Nonlocal node"""
 
-class ListComp(NodeNG):
-    """class representing a ListComp node"""
-    _astng_fields = ('elt', 'generators')
-    elt = None
-    generators = None
+    def __init__(self, names):
+        self.names = names
+
+    def _infer_name(self, frame, name):
+        return name
 
 
 class Pass(StmtMixIn, NodeNG):
@@ -691,12 +681,25 @@ class Print(StmtMixIn, NodeNG):
     values = None
 
 
+
 class Raise(StmtMixIn, NodeNG):
     """class representing a Raise node"""
-    _astng_fields = ('type', 'inst', 'tback')
-    type = None
-    inst = None
-    tback = None
+    exc = None
+    if sys.version_info < (3, 0):
+        _astng_fields = ('exc', 'inst', 'tback')
+        inst = None
+        tback = None
+    else:
+        _astng_fields = ('exc', 'cause')
+        inst = None
+        tback = None
+
+    def raises_not_implemented(self):
+        if not self.exc:
+            return
+        for name in self.exc.nodes_of_class(Name):
+            if name.name == 'NotImplementedError':
+                return True
 
 
 class Return(StmtMixIn, NodeNG):
@@ -705,6 +708,18 @@ class Return(StmtMixIn, NodeNG):
     value = None
 
 
+class Set(NodeNG, Instance, ParentAssignTypeMixin):
+    """class representing a Set node"""
+    _astng_fields = ('elts',)
+    elts = None
+
+    def pytype(self):
+        return '__builtin__.set' # XXX __builtin__ vs builtins
+
+    def itered(self):
+        return self.elts
+
+
 class Slice(NodeNG):
     """class representing a Slice node"""
     _astng_fields = ('lower', 'upper', 'step')
@@ -712,6 +727,11 @@ class Slice(NodeNG):
     upper = None
     step = None
 
+class Starred(NodeNG):
+    """class representing a Starred node"""
+    _astng_fields = ('value',)
+    value = None
+
 
 class Subscript(NodeNG):
     """class representing a Subscript node"""
