@@ -1,15 +1,3 @@
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU Lesser General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # copyright 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
 # copyright 2003-2010 Sylvain Thenault, all rights reserved.
@@ -99,7 +87,7 @@ def const_infer_binary_op(self, operator, other, context):
 
                 try:
                     yield const_factory(impl(self.value, other.value))
-                except StandardError:
+                except Exception:
                     # ArithmeticError is not enough: float >> float is a TypeError
                     # TODO : let pylint know about the problem
                     pass
@@ -128,6 +116,9 @@ def tl_infer_binary_op(self, operator, other, context):
             node.elts = elts
             yield node
         elif isinstance(other, nodes.Const) and operator == '*':
+            if not isinstance(other.value, int):
+                yield YES
+                continue
             node = self.__class__()
             elts = [n for elt in self.elts for n in elt.infer(context)
                     if not n is YES] * other.value
