@@ -75,6 +75,9 @@ REDIRECT = {'arguments': 'Arguments',
             'Invert': 'UnaryOp'
             }
 
+_key_func = lambda node: node.fromlineno
+def sort_locals(my_list):
+    my_list.sort(key=_key_func)
 
 class RebuildVisitor(object):
     """Visitor to transform an AST to an ASTNG
@@ -94,14 +97,14 @@ class RebuildVisitor(object):
         visit_method = getattr(self, visit_name)
         return visit_method(node, parent)
 
-    def build(self, node, modname, module_file):
+    def build(self, node, modname, module_file, package):
         """rebuild the tree starting with an Module node;
         return an astng.Module node
         """
         self._metaclass = ['']
         self._global_names = []
         self._from_nodes = []
-        module = self.visit_module(node, modname)
+        module = self.visit_module(node, modname, package)
         module.file = module.path = module_file
         # init module cache here else we may get some infinite recursion
         # errors while infering delayed assignments
@@ -204,7 +207,6 @@ class RebuildVisitor(object):
         """store imported names to the locals;
         resort the locals if coming from a delayed node
         """
-        cmp_nodes = lambda x, y: cmp(x.fromlineno, y.fromlineno)
         for (name, asname) in node.names:
             if name == '*':
                 try:
@@ -214,11 +216,12 @@ class RebuildVisitor(object):
                 for name in imported.wildcard_import_names():
                     node.parent.set_local(name, node)
                     if delayed:
-                        node.parent.scope().locals[name].sort(cmp=cmp_nodes)
+                        sort_locals(node.parent.scope().locals[name])
+
             else:
                 node.parent.set_local(asname or name, node)
                 if delayed:
-                    node.parent.scope().locals[asname or name].sort(cmp=cmp_nodes)
+                    sort_locals(node.parent.scope().locals[asname or name])
 
 
     def visit_function(self, node, parent):
