@@ -32,18 +32,20 @@
 """tests for the astng variable lookup capabilities
 """
 import sys
-from os.path import join, abspath
+from os.path import join, abspath, dirname
 from logilab.common.testlib import TestCase, unittest_main
 
 from logilab.astng import builder, nodes, scoped_nodes, \
-     InferenceError, NotFoundError
+     InferenceError, NotFoundError, UnresolvableName
 from logilab.astng.scoped_nodes import builtin_lookup
+from logilab.astng.bases import YES
 from unittest_inference import get_name_node
 
 builder = builder.ASTNGBuilder()
-MODULE = builder.file_build('data/module.py', 'data.module')
-MODULE2 = builder.file_build('data/module2.py', 'data.module2')
-NONREGR = builder.file_build('data/nonregr.py', 'data.nonregr')
+DATA = join(dirname(abspath(__file__)), 'data')
+MODULE = builder.file_build(join(DATA, 'module.py'), 'data.module')
+MODULE2 = builder.file_build(join(DATA, 'module2.py'), 'data.module2')
+NONREGR = builder.file_build(join(DATA, 'nonregr.py'), 'data.nonregr')
 
 class LookupTC(TestCase):
 
@@ -60,16 +62,24 @@ def func():
     c = 1
         '''
         astng = builder.string_build(code, __name__, __file__)
-        names = astng.nodes_of_class(nodes.Name)
-        a = names.next()
+        # a & b
+        a = astng.nodes_of_class(nodes.Name).next()
+        self.assertEqual(a.lineno, 2)
+        if sys.version_info < (3, 0):
+            self.failUnlessEqual(len(astng.lookup('b')[1]), 2)
+            self.failUnlessEqual(len(astng.lookup('a')[1]), 3)
+            b = astng.locals['b'][1]
+        else:
+            self.failUnlessEqual(len(astng.lookup('b')[1]), 1)
+            self.failUnlessEqual(len(astng.lookup('a')[1]), 2)
+            b = astng.locals['b'][0]
         stmts = a.lookup('a')[1]
         self.failUnlessEqual(len(stmts), 1)
-        b = astng.locals['b'][1]
-        #self.failUnlessEqual(len(b.lookup('b')[1]), 1)
-        self.failUnlessEqual(len(astng.lookup('b')[1]), 2)
+        self.assertEqual(b.lineno, 6)
         b_infer = b.infer()
         b_value = b_infer.next()
         self.failUnlessEqual(b_value.value, 1)
+        # c
         self.failUnlessRaises(StopIteration, b_infer.next)
         func = astng.locals['func'][0]
         self.failUnlessEqual(len(func.lookup('c')[1]), 1)
@@ -78,14 +88,14 @@ def func():
         astng = builder.string_build('pass', __name__, __file__)
         # built-in objects
         none = astng.ilookup('None').next()
-        self.assertEquals(none.value, None)
+        self.assertEqual(none.value, None)
         obj = astng.ilookup('object').next()
         self.assertIsInstance(obj, nodes.Class)
-        self.assertEquals(obj.name, 'object')
+        self.assertEqual(obj.name, 'object')
         self.assertRaises(InferenceError, astng.ilookup('YOAA').next)
 
         # XXX
-        self.assertEquals(len(list(NONREGR.ilookup('enumerate'))), 2)
+        self.assertEqual(len(list(NONREGR.ilookup('enumerate'))), 2)
 
     def test_class_ancestor_name(self):
         code = '''
@@ -99,7 +109,7 @@ class A(A):
         cls1 = astng.locals['A'][0]
         cls2 = astng.locals['A'][1]
         name = cls2.nodes_of_class(nodes.Name).next()
-        self.assertEquals(name.infer().next(), cls1)
+        self.assertEqual(name.infer().next(), cls1)
         
     ### backport those test to inline code
     def test_method(self):
@@ -107,7 +117,7 @@ class A(A):
         my_dict = method.ilookup('MY_DICT').next()
         self.assert_(isinstance(my_dict, nodes.Dict), my_dict)
         none = method.ilookup('None').next()
-        self.assertEquals(none.value, None)
+        self.assertEqual(none.value, None)
         self.assertRaises(InferenceError, method.ilookup('YOAA').next)
 
         
@@ -115,8 +125,8 @@ class A(A):
         make_class = MODULE2['make_class']
         base = make_class.ilookup('base').next()
         self.assert_(isinstance(base, nodes.Class), base.__class__)
-        self.assertEquals(base.name, 'YO')
-        self.assertEquals(base.root().name, 'data.module')
+        self.assertEqual(base.name, 'YO')
+        self.assertEqual(base.root().name, 'data.module')
 
 
     def test_class(self):
@@ -124,49 +134,104 @@ class A(A):
         my_dict = klass.ilookup('MY_DICT').next()
         self.assertIsInstance(my_dict, nodes.Dict)
         none = klass.ilookup('None').next()
-        self.assertEquals(none.value, None)
+        self.assertEqual(none.value, None)
         obj = klass.ilookup('object').next()
         self.assertIsInstance(obj, nodes.Class)
-        self.assertEquals(obj.name, 'object')
+        self.assertEqual(obj.name, 'object')
         self.assertRaises(InferenceError, klass.ilookup('YOAA').next)
 
 
     def test_inner_classes(self):
-        ccc = NONREGR['Ccc']
-        self.assertEquals(ccc.ilookup('Ddd').next().name, 'Ddd')
+        ddd = list(NONREGR['Ccc'].ilookup('Ddd'))
+        self.assertEqual(ddd[0].name, 'Ddd')
 
 
     def test_loopvar_hiding(self):
         astng = builder.string_build("""
 x = 10
 for x in range(5):
-    print x
+    print (x)
    
 if x > 0:
-    print '#' * x        
+    print ('#' * x)
         """, __name__, __file__)
         xnames = [n for n in astng.nodes_of_class(nodes.Name) if n.name == 'x']
         # inside the loop, only one possible assignment
-        self.assertEquals(len(xnames[0].lookup('x')[1]), 1)
+        self.assertEqual(len(xnames[0].lookup('x')[1]), 1)
         # outside the loop, two possible assignments
-        self.assertEquals(len(xnames[1].lookup('x')[1]), 2)
-        self.assertEquals(len(xnames[2].lookup('x')[1]), 2)
+        self.assertEqual(len(xnames[1].lookup('x')[1]), 2)
+        self.assertEqual(len(xnames[2].lookup('x')[1]), 2)
 
     def test_list_comps(self):
-        if sys.version_info < (2, 4):
-            self.skip('this test require python >= 2.4')
         astng = builder.string_build("""
-print [ i for i in range(10) ]
-print [ i for i in range(10) ]
-print list( i for i in range(10) )
+print ([ i for i in range(10) ])
+print ([ i for i in range(10) ])
+print ( list( i for i in range(10) ) )
         """, __name__, __file__)
         xnames = [n for n in astng.nodes_of_class(nodes.Name) if n.name == 'i']
-        self.assertEquals(len(xnames[0].lookup('i')[1]), 1)
-        self.assertEquals(xnames[0].lookup('i')[1][0].lineno, 2)
-        self.assertEquals(len(xnames[1].lookup('i')[1]), 1)
-        self.assertEquals(xnames[1].lookup('i')[1][0].lineno, 3)
-        self.assertEquals(len(xnames[2].lookup('i')[1]), 1)
-        self.assertEquals(xnames[2].lookup('i')[1][0].lineno, 4)
+        self.assertEqual(len(xnames[0].lookup('i')[1]), 1)
+        self.assertEqual(xnames[0].lookup('i')[1][0].lineno, 2)
+        self.assertEqual(len(xnames[1].lookup('i')[1]), 1)
+        self.assertEqual(xnames[1].lookup('i')[1][0].lineno, 3)
+        self.assertEqual(len(xnames[2].lookup('i')[1]), 1)
+        self.assertEqual(xnames[2].lookup('i')[1][0].lineno, 4)
+
+    def test_list_comp_target(self):
+        """test the list comprehension target"""
+        astng = builder.string_build("""
+ten = [ var for var in range(10) ]
+var
+        """)
+        var = astng.body[1].value
+        if sys.version_info < (3, 0):
+            self.assertEqual(var.infered(), [YES])
+        else:
+            self.assertRaises(UnresolvableName, var.infered)
+
+
+    def test_dict_comps(self):
+        if sys.version_info < (2, 7):
+            self.skipTest('this test require python >= 2.7')
+        astng = builder.string_build("""
+print ({ i: j for i in range(10) for j in range(10) })
+print ({ i: j for i in range(10) for j in range(10) })
+        """, __name__, __file__)
+        xnames = [n for n in astng.nodes_of_class(nodes.Name) if n.name == 'i']
+        self.assertEqual(len(xnames[0].lookup('i')[1]), 1)
+        self.assertEqual(xnames[0].lookup('i')[1][0].lineno, 2)
+        self.assertEqual(len(xnames[1].lookup('i')[1]), 1)
+        self.assertEqual(xnames[1].lookup('i')[1][0].lineno, 3)
+
+        xnames = [n for n in astng.nodes_of_class(nodes.Name) if n.name == 'j']
+        self.assertEqual(len(xnames[0].lookup('i')[1]), 1)
+        self.assertEqual(xnames[0].lookup('i')[1][0].lineno, 2)
+        self.assertEqual(len(xnames[1].lookup('i')[1]), 1)
+        self.assertEqual(xnames[1].lookup('i')[1][0].lineno, 3)
+
+
+    def test_set_comps(self):
+        if sys.version_info < (2, 7):
+            self.skipTest('this test require python >= 2.7')
+        astng = builder.string_build("""
+print ({ i for i in range(10) })
+print ({ i for i in range(10) })
+        """, __name__, __file__)
+        xnames = [n for n in astng.nodes_of_class(nodes.Name) if n.name == 'i']
+        self.assertEqual(len(xnames[0].lookup('i')[1]), 1)
+        self.assertEqual(xnames[0].lookup('i')[1][0].lineno, 2)
+        self.assertEqual(len(xnames[1].lookup('i')[1]), 1)
+        self.assertEqual(xnames[1].lookup('i')[1][0].lineno, 3)
+
+    def test_set_comp_closure(self):
+        if sys.version_info < (2, 7):
+            self.skipTest('this test require python >= 2.7')
+        astng = builder.string_build("""
+ten = { var for var in range(10) }
+var
+        """)
+        var = astng.body[1].value
+        self.assertRaises(UnresolvableName, var.infered)
+
 
 
     def test_explicit___name__(self):
@@ -198,46 +263,42 @@ def initialize(linter):
     package_load(linter, __path__[0])
         ''', 'data.__init__', 'data/__init__.py')
         path = [n for n in astng.nodes_of_class(nodes.Name) if n.name == '__path__'][0]
-        self.assertEquals(len(path.lookup('__path__')[1]), 1)
+        self.assertEqual(len(path.lookup('__path__')[1]), 1)
 
 
     def test_builtin_lookup(self):
-        self.assertEquals(builtin_lookup('__dict__')[1], ())
+        self.assertEqual(builtin_lookup('__dict__')[1], ())
         intstmts = builtin_lookup('int')[1]
-        self.assertEquals(len(intstmts), 1)
+        self.assertEqual(len(intstmts), 1)
         self.assertIsInstance(intstmts[0], nodes.Class)
-        self.assertEquals(intstmts[0].name, 'int')
+        self.assertEqual(intstmts[0].name, 'int')
         self.assertIs(intstmts[0], nodes.const_factory(1)._proxied)
 
 
     def test_decorator_arguments_lookup(self):
-        if sys.version_info < (2, 4):
-            self.skip('this test require python >= 2.4')
         code = '''
 def decorator(value):
-   def wrapper(function):
+    def wrapper(function):
         return function
-   return wrapper
+    return wrapper
 
 class foo:
-  member = 10
+    member = 10
 
-  @decorator(member) #This will cause pylint to complain
-  def test(self):
-       pass
+    @decorator(member) #This will cause pylint to complain
+    def test(self):
+        pass
         ''' 
         astng = builder.string_build(code, __name__, __file__)
         member = get_name_node(astng['foo'], 'member')
         it = member.infer()
         obj = it.next()
         self.assertIsInstance(obj, nodes.Const)
-        self.assertEquals(obj.value, 10)
+        self.assertEqual(obj.value, 10)
         self.assertRaises(StopIteration, it.next)
 
        
     def test_inner_decorator_member_lookup(self):
-        if sys.version_info < (2, 4):
-            self.skip('this test require python >= 2.4')
         code = '''
 class FileA:
     def decorator(bla):
@@ -256,8 +317,6 @@ class FileA:
         
         
     def test_static_method_lookup(self):
-        if sys.version_info < (2, 4):
-            self.skip('this test require python >= 2.4')
         code = '''
 class FileA:
     @staticmethod
@@ -269,7 +328,7 @@ class Test:
     FileA = [1,2,3]
     
     def __init__(self):
-        print FileA.funcA()
+        print (FileA.funcA())
         '''
         astng = builder.string_build(code, __name__, __file__)
         it = astng['Test']['__init__'].ilookup('FileA')
@@ -284,7 +343,7 @@ def run2():
     f = Frobble()
     
 class Frobble:
-     pass
+    pass
 Frobble.mumble = True
 
 del Frobble
