@@ -35,10 +35,14 @@ import sys
 
 from logilab.common import testlib
 from logilab.astng import builder, nodes, NotFoundError
-from logilab.astng.nodes_as_string import as_string
+from logilab.astng.as_string import as_string
 
 from data import module as test_module
 
+from os.path import join, abspath, dirname
+
+DATA = join(dirname(abspath(__file__)), 'data')
+
 abuilder = builder.ASTNGBuilder()
 
 class _NodeTC(testlib.TestCase):
@@ -58,22 +62,22 @@ class IfNodeTC(_NodeTC):
     """test transformation of If Node"""
     CODE = """
 if 0:
-    print
+    print()
 
 if True:
-    print
+    print()
 else:
     pass
 
 if "":
-    print
+    print()
 elif []:
     raise
 
 if 1:
-    print
+    print()
 elif True:
-    print
+    print()
 elif func():
     pass
 else:
@@ -82,7 +86,7 @@ else:
         
     def test_if_elif_else_node(self):
         """test transformation for If node"""
-        self.assertEquals(len(self.astng.body), 4)
+        self.assertEqual(len(self.astng.body), 4)
         for stmt in self.astng.body:
             self.assertIsInstance( stmt, nodes.If)
         self.failIf(self.astng.body[0].orelse) # simple If
@@ -92,75 +96,87 @@ else:
 
     def test_block_range(self):
         # XXX ensure expected values
-        self.assertEquals(self.astng.block_range(1), (0, 22))
-        self.assertEquals(self.astng.block_range(10), (0, 22)) # XXX (10, 22) ?
-        self.assertEquals(self.astng.body[1].block_range(5), (5, 6))
-        self.assertEquals(self.astng.body[1].block_range(6), (6, 6))
-        self.assertEquals(self.astng.body[1].orelse[0].block_range(7), (7, 8))
-        self.assertEquals(self.astng.body[1].orelse[0].block_range(8), (8, 8))
+        self.assertEqual(self.astng.block_range(1), (0, 22))
+        self.assertEqual(self.astng.block_range(10), (0, 22)) # XXX (10, 22) ?
+        self.assertEqual(self.astng.body[1].block_range(5), (5, 6))
+        self.assertEqual(self.astng.body[1].block_range(6), (6, 6))
+        self.assertEqual(self.astng.body[1].orelse[0].block_range(7), (7, 8))
+        self.assertEqual(self.astng.body[1].orelse[0].block_range(8), (8, 8))
 
 
 class TryExceptNodeTC(_NodeTC):
     CODE = """
 try:
-    print 'pouet'
+    print ('pouet')
 except IOError:
     pass
 except UnicodeError:
-    print
+    print()
 else:
-    print
+    print()
     """
     def test_block_range(self):
         # XXX ensure expected values
-        self.assertEquals(self.astng.body[0].block_range(1), (1, 8))
-        self.assertEquals(self.astng.body[0].block_range(2), (2, 2))
-        self.assertEquals(self.astng.body[0].block_range(3), (3, 8))
-        self.assertEquals(self.astng.body[0].block_range(4), (4, 4))
-        self.assertEquals(self.astng.body[0].block_range(5), (5, 5))
-        self.assertEquals(self.astng.body[0].block_range(6), (6, 6))
-        self.assertEquals(self.astng.body[0].block_range(7), (7, 7))
-        self.assertEquals(self.astng.body[0].block_range(8), (8, 8))
+        self.assertEqual(self.astng.body[0].block_range(1), (1, 8))
+        self.assertEqual(self.astng.body[0].block_range(2), (2, 2))
+        self.assertEqual(self.astng.body[0].block_range(3), (3, 8))
+        self.assertEqual(self.astng.body[0].block_range(4), (4, 4))
+        self.assertEqual(self.astng.body[0].block_range(5), (5, 5))
+        self.assertEqual(self.astng.body[0].block_range(6), (6, 6))
+        self.assertEqual(self.astng.body[0].block_range(7), (7, 7))
+        self.assertEqual(self.astng.body[0].block_range(8), (8, 8))
 
 
 class TryFinallyNodeTC(_NodeTC):
     CODE = """
 try:
-    print 'pouet'
+    print ('pouet')
 finally:
-    print 'pouet'
+    print ('pouet')
     """
     def test_block_range(self):
         # XXX ensure expected values
-        self.assertEquals(self.astng.body[0].block_range(1), (1, 4))
-        self.assertEquals(self.astng.body[0].block_range(2), (2, 2))
-        self.assertEquals(self.astng.body[0].block_range(3), (3, 4))
-        self.assertEquals(self.astng.body[0].block_range(4), (4, 4))
+        self.assertEqual(self.astng.body[0].block_range(1), (1, 4))
+        self.assertEqual(self.astng.body[0].block_range(2), (2, 2))
+        self.assertEqual(self.astng.body[0].block_range(3), (3, 4))
+        self.assertEqual(self.astng.body[0].block_range(4), (4, 4))
 
 
 class TryFinally25NodeTC(_NodeTC):
     CODE = """
 try:
-    print 'pouet'
+    print('pouet')
 except Exception:
-    print 'oops'
+    print ('oops')
 finally:
-    print 'pouet'
+    print ('pouet')
     """
     def test_block_range(self):
-        if sys.version_info < (2, 5):
-            self.skip('require python >= 2.5')
         # XXX ensure expected values
-        self.assertEquals(self.astng.body[0].block_range(1), (1, 6))
-        self.assertEquals(self.astng.body[0].block_range(2), (2, 2))
-        self.assertEquals(self.astng.body[0].block_range(3), (3, 4))
-        self.assertEquals(self.astng.body[0].block_range(4), (4, 4))
-        self.assertEquals(self.astng.body[0].block_range(5), (5, 5))
-        self.assertEquals(self.astng.body[0].block_range(6), (6, 6))
+        self.assertEqual(self.astng.body[0].block_range(1), (1, 6))
+        self.assertEqual(self.astng.body[0].block_range(2), (2, 2))
+        self.assertEqual(self.astng.body[0].block_range(3), (3, 4))
+        self.assertEqual(self.astng.body[0].block_range(4), (4, 4))
+        self.assertEqual(self.astng.body[0].block_range(5), (5, 5))
+        self.assertEqual(self.astng.body[0].block_range(6), (6, 6))
+
+
+class TryExcept2xNodeTC(_NodeTC):
+    CODE = """
+try:
+    hello
+except AttributeError, (retval, desc):
+    pass
+    """
+    def test_tuple_attribute(self):
+        if sys.version_info >= (3, 0):
+            self.skipTest('syntax removed from py3.x')
+        handler = self.astng.body[0].handlers[0]
+        self.assertIsInstance(handler.name, nodes.Tuple)
+
 
-        
 MODULE = abuilder.module_build(test_module)
-MODULE2 = abuilder.file_build('data/module2.py', 'data.module2')
+MODULE2 = abuilder.file_build(join(DATA, 'module2.py'), 'data.module2')
 
 
 class ImportNodeTC(testlib.TestCase):
@@ -186,37 +202,76 @@ class ImportNodeTC(testlib.TestCase):
 
     def test_real_name(self):
         from_ = MODULE['spawn']
-        self.assertEquals(from_.real_name('spawn'), 'Execute')
+        self.assertEqual(from_.real_name('spawn'), 'Execute')
         imp_ = MODULE['os']
-        self.assertEquals(imp_.real_name('os'), 'os')
+        self.assertEqual(imp_.real_name('os'), 'os')
         self.assertRaises(NotFoundError, imp_.real_name, 'os.path')
         imp_ = MODULE['spawn']
-        self.assertEquals(imp_.real_name('spawn'), 'Execute')
+        self.assertEqual(imp_.real_name('spawn'), 'Execute')
         self.assertRaises(NotFoundError, imp_.real_name, 'Execute')
         imp_ = MODULE2['YO']
-        self.assertEquals(imp_.real_name('YO'), 'YO')
+        self.assertEqual(imp_.real_name('YO'), 'YO')
         self.assertRaises(NotFoundError, imp_.real_name, 'data')
 
     def test_as_string(self):
         
         ast = MODULE['modutils']
-        self.assertEquals(as_string(ast), "from logilab.common import modutils")
+        self.assertEqual(as_string(ast), "from logilab.common import modutils")
         ast = MODULE['spawn']
-        self.assertEquals(as_string(ast), "from logilab.common.shellutils import Execute as spawn")
+        self.assertEqual(as_string(ast), "from logilab.common.shellutils import Execute as spawn")
         ast = MODULE['os']
-        self.assertEquals(as_string(ast), "import os.path")
-    
+        self.assertEqual(as_string(ast), "import os.path")
+        ast = abuilder.string_build( 'raise_string(*args, **kwargs)').body[0]
+        self.assertEqual(as_string(ast), 'raise_string(*args, **kwargs)')
+
     def test_module_as_string(self):
-        """just check as_string on a whole module doesn't raise an exception
+        """check as_string on a whole module prepared to be returned identically
         """
-        self.assert_(as_string(MODULE))
-        self.assert_(as_string(MODULE2))
-        
+        data = open(join(DATA, 'module.py')).read()
+        self.assertMultiLineEqual(as_string(MODULE), data)
+        data = open(join(DATA, 'module2.py')).read()
+        self.assertMultiLineEqual(as_string(MODULE2), data)
+
+    def test_2_7_as_string(self):
+        """check as_string for python syntax >= 2.7"""
+        if sys.version_info < (2, 7):
+            self.skipTest("test python >= 2.7 specific")
+        code = '''one_two = {1, 2}
+b = {v: k for (k, v) in enumerate('string')}
+cdd = {k for k in b}\n\n'''
+        ast = abuilder.string_build(code)
+        self.assertMultiLineEqual(as_string(ast), code)
+
+    def test_3k_as_string(self):
+        """check as_string for python 3k syntax"""
+        if sys.version_info < (3, 0):
+            self.skipTest("test python 3k specific")
+        code = '''print()
+
+def function(var):
+    nonlocal counter
+    try:
+        hello
+    except NameError as nexc:
+        (*hell, o) = b'hello'
+        raise AttributeError from nexc
+\n'''
+        # TODO : annotations and keywords for class definition are not yet implemented
+        _todo = '''
+def function(var:int):
+    nonlocal counter
+
+class Language(metaclass=Natural):
+    """natural language"""
+        '''
+        ast = abuilder.string_build(code)
+        self.assertEqual(as_string(ast), code)
+
 
 class CmpNodeTC(testlib.TestCase):
     def test_as_string(self):
-        ast = abuilder.string_build("a == 2")
-        self.assertEquals(as_string(ast), "a == 2")
+        ast = abuilder.string_build("a == 2").body[0]
+        self.assertEqual(as_string(ast), "a == 2")
 
 
 class ConstNodeTC(testlib.TestCase):
@@ -224,10 +279,10 @@ class ConstNodeTC(testlib.TestCase):
     def _test(self, value):
         node = nodes.const_factory(value)
         self.assertIsInstance(node._proxied, nodes.Class)
-        self.assertEquals(node._proxied.name, value.__class__.__name__)
+        self.assertEqual(node._proxied.name, value.__class__.__name__)
         self.assertIs(node.value, value)
         self.failUnless(node._proxied.parent)
-        self.assertEquals(node._proxied.root().name, value.__class__.__module__)
+        self.assertEqual(node._proxied.root().name, value.__class__.__module__)
         
     def test_none(self):
         self._test(None)
@@ -258,12 +313,12 @@ def func(a,
     b): pass
 x = lambda x: None
         ''')
-        self.assertEquals(ast['func'].args.fromlineno, 2)
-        self.assertEquals(ast['func'].args.tolineno, 3)
+        self.assertEqual(ast['func'].args.fromlineno, 2)
+        self.assertEqual(ast['func'].args.tolineno, 3)
         self.failIf(ast['func'].args.is_statement)
         xlambda = ast['x'].infer().next()
-        self.assertEquals(xlambda.args.fromlineno, 4)
-        self.assertEquals(xlambda.args.tolineno, 4)
+        self.assertEqual(xlambda.args.fromlineno, 4)
+        self.assertEqual(xlambda.args.tolineno, 4)
         self.failIf(xlambda.args.is_statement)
 
 
@@ -271,8 +326,8 @@ class SliceNodeTC(testlib.TestCase):
     def test(self):
         for code in ('a[0]', 'a[1:3]', 'a[:-1:step]', 'a[:,newaxis]',
                      'a[newaxis,:]', 'del L[::2]', 'del A[1]', 'del Br[:]'):
-            ast = abuilder.string_build(code)
-            self.assertEquals(ast.as_string(), code)
+            ast = abuilder.string_build(code).body[0]
+            self.assertEqual(ast.as_string(), code)
 
     def test_slice_and_subscripts(self):
         code = """a[:1] = bord[2:]
@@ -289,14 +344,14 @@ del Fee.form[left:]
 aout.vals = miles.of_stuff
 del (ccok, (name.thing, foo.attrib.value)), Fee.form[left:]
 if all[1] == bord[0:]:
-    pass"""
+    pass\n\n"""
         ast = abuilder.string_build(code)
-        self.assertEquals(ast.as_string(), code)
+        self.assertEqual(ast.as_string(), code)
 
 class EllipsisNodeTC(testlib.TestCase):
     def test(self):
-        ast = abuilder.string_build('a[...]')
-        self.assertEquals(ast.as_string(), 'a[...]')
+        ast = abuilder.string_build('a[...]').body[0]
+        self.assertEqual(ast.as_string(), 'a[...]')
         
 if __name__ == '__main__':
     testlib.unittest_main()
