@@ -38,7 +38,6 @@ __docformat__ = "restructuredtext en"
 
 from logilab.astng._exceptions import IgnoreChild, ASTNGBuildingException
 
-
 class ASTVisitor(object):
     """Abstract Base Class for Python AST Visitors.
 
@@ -114,6 +113,9 @@ class ASTVisitor(object):
     def visit_dict(self, node):
         """dummy method for visiting an Dict node"""
 
+    def visit_dictcomp(self, node):
+        """dummy method for visiting an DictComp node"""
+
     def visit_discard(self, node):
         """dummy method for visiting an Discard node"""
 
@@ -195,6 +197,9 @@ class ASTVisitor(object):
     def visit_return(self, node):
         """dummy method for visiting an Return node"""
 
+    def visit_setcomp(self, node):
+        """dummy method for visiting an SetComp node"""
+
     def visit_slice(self, node):
         """dummy method for visiting an Slice node"""
 
@@ -297,7 +302,7 @@ class LocalsVisitor(ASTWalker):
 
     def visit(self, node):
         """launch the visit starting from the given node"""
-        if self._visited.has_key(node):
+        if node in self._visited:
             return
         self._visited[node] = 1 # FIXME: use set ?
         methods = self.get_callbacks(node)
@@ -338,5 +343,115 @@ def _check_children(node):
         _check_children(child)
 
 
+# XXX This class don't support 'compiler' for now.
+# If we want it, there is a "_native_repr_tree" method in _nodes_compiler.py
+class TreeTester(object):
+    '''A helper class to see _ast tree and compare with astng tree
+
+    indent: string for tree indent representation
+    lineno: bool to tell if we should print the line numbers
+
+    >>> tester = TreeTester('print')
+    >>> print tester.native_tree_repr()
+
+    <Module>
+    .   body = [
+    .   <Print>
+    .   .   nl = True
+    .   ]
+    >>> print tester.astng_tree_repr()
+    Module()
+        body = [
+        Print()
+            dest = 
+            values = [
+            ]
+        ]
+    '''
+
+    indent = '.   '
+    lineno = False
+
+    def __init__(self, sourcecode):
+        self._string = ''
+        self.sourcecode = sourcecode
+        self._ast_node = None
+        self.build_ast()
+
+    def build_ast(self):
+        """build the _ast tree from the source code"""
+        from _ast import PyCF_ONLY_AST
+        def parse(string):
+            return compile(string, "<string>", 'exec', PyCF_ONLY_AST)
+        self._ast_node = parse(self.sourcecode)
+
+    def native_tree_repr(self, node=None, indent=''):
+        """get a nice representation of the _ast tree"""
+        self._string = ''
+        if node is None:
+            node = self._ast_node
+        self._native_repr_tree(node, indent)
+        return self._string
+
+
+    def _native_repr_tree(self, node, indent, _done=None):
+        """recursive method for the native tree representation"""
+        from _ast import Load as _Load, Store as _Store, Del as _Del
+        from _ast import AST as Node
+        if _done is None:
+            _done = set()
+        if node in _done:
+            self._string += '\nloop in tree: %r (%s)' % (node,
+                                            getattr(node, 'lineno', None))
+            return
+        _done.add(node)
+        self._string += '\n' + indent +  '<%s>' % node.__class__.__name__
+        indent += self.indent
+        if not hasattr(node, '__dict__'):
+            self._string += '\n' + self.indent + " ** node has no __dict__ " + str(node)
+            return
+        node_dict = node.__dict__
+        if hasattr(node, '_attributes'):
+            for a in node._attributes:
+                attr = node_dict[a]
+                if attr is None:
+                    continue
+                if a in ("lineno", "col_offset") and not self.lineno:
+                    continue
+                self._string +='\n' +  indent + a + " = " + repr(attr)
+        for field in node._fields or ():
+            attr = node_dict[field]
+            if attr is None:
+                continue
+            if isinstance(attr, list):
+                if not attr:
+                    continue
+                self._string += '\n' + indent + field + ' = ['
+                for elt in attr:
+                    self._native_repr_tree(elt, indent, _done)
+                self._string += '\n' + indent + ']'
+                continue
+            if isinstance(attr, (_Load, _Store, _Del)):
+                continue
+            if isinstance(attr, Node):
+                self._string += '\n' + indent + field + " = "
+                self._native_repr_tree(attr, indent, _done)
+            else:
+                self._string += '\n' + indent + field + " = " + repr(attr)
+
+
+    def build_astng_tree(self):
+        """build astng tree from the _ast tree
+        """
+        from logilab.astng.builder import ASTNGBuilder
+        tree = ASTNGBuilder().ast_build(self._ast_node)
+        return tree
+
+    def astng_tree_repr(self, ids=False):
+        """build the astng tree and return a nice tree representation"""
+        mod = self.build_astng_tree()
+        return mod.repr_tree(ids)
+
+
 __all__ = ('LocalsVisitor', 'ASTWalker', 'ASTVisitor',)
 
